@@ -1,10 +1,70 @@
 
+import functools
+from collections import namedtuple
+
+import sqlalchemy
 from sqlalchemy import inspect, orm, and_, or_
-from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.ext.declarative import declarative_base, DeclarativeMeta
 from sqlalchemy.orm.exc import UnmappedClassError
 
 import query
-from utils import classproperty, is_sequence
+from utils import classproperty, is_sequence, has_primary_key, camelcase_to_underscore
+
+Event = namedtuple('Event', ['name', 'listener', 'kargs'])
+
+class ModelMeta(DeclarativeMeta):
+    def __new__(cls, name, bases, dict_):
+        # set __tablename__ (if not defined) to underscore version of class name
+        if not dict_.get('__tablename__') and not dict_.get('__table__') is not None and has_primary_key(dict_):
+            dict_['__tablename__'] = camelcase_to_underscore(name)
+
+        # set __events__ to expected default so that it's updatable when initializing
+        # e.g. if class definition sets __events__=None but defines decorated events,
+        # then we want the final __events__ attribute to reflect the registered events.
+        # if set to anything that's non-empty/non-dict will lead to an error if decorated events defined
+        if not dict_.get('__events__'):
+            dict_['__events__'] = {}
+
+        return DeclarativeMeta.__new__(cls, name, bases, dict_)
+
+    def __init__(cls, name, bases, dict_):
+        events = []
+
+        # append class attribute defined events
+        if dict_.get('__events__'):
+            # events defined on __events__ can have many forms (e.g. string based, list of tuples, etc)
+            # so we need to iterate over them and parse into standardized Event object
+            for event_name, listeners in dict_['__events__'].iteritems():
+                if not isinstance(listeners, list):
+                    listeners = [listeners]
+
+                for listener in listeners:
+                    if isinstance(listener, tuple):
+                        # listener definition includes event.listen keyword args
+                        listener, kargs = listener
+                    else:
+                        kargs = {}
+
+                    if not hasattr(listener, '__call__'):
+                        # no __call__ attribute? then assume listener is a string reference to class method
+                        listener = dict_[listener]
+
+                    events.append(Event(event_name, listener, kargs))
+
+        # append events which were added via @event decorator
+        events += [value.__event__ for value in dict_.values() if hasattr(value, '__event__')]
+
+        if events:
+            # reassemble events dict into consistent form using Event objects as values
+            events_dict = {}
+            for event in events:
+                sqlalchemy.event.listen(cls, event.name, event.listener, **event.kargs)
+                events_dict.setdefault(event.name, []).append(event)
+
+            dict_['__events__'].update(events_dict)
+
+        super(ModelMeta, cls).__init__(name, bases, dict_)
+
 
 class ModelBase(object):
     '''Augmentable Base class for adding shared model properties/functions'''
@@ -16,6 +76,9 @@ class ModelBase(object):
     # eg: { 'order_by': [column1, column2] }
     __mapper_args__ = {}
 
+    # register orm event listeners
+    __events__ = {}
+
     # query class to use for self.query
     query_class = query.Query
 
@@ -27,8 +90,8 @@ class ModelBase(object):
 
     # specify sqla.session.query.filter() options for advanced and simple searches
     # eg: { key: lambda value: Model.column_name == val }
-    advanced_search_config = {}
-    simple_search_config = {}
+    __advanced_search__ = {}
+    __simple_search__ = {}
 
     def __init__(self, *args, **kargs):
         self.update(*args, **kargs)
@@ -116,8 +179,8 @@ class ModelBase(object):
     @classmethod
     def advanced_search(cls, search_dict):
         filters = None
-        if cls.advanced_search_config:
-            _filters = cls.get_search(search_dict, cls.advanced_search_config)
+        if cls.__advanced_search__:
+            _filters = cls.get_search(search_dict, cls.__advanced_search__)
 
             if _filters:
                 filters = and_(*_filters)
@@ -128,9 +191,9 @@ class ModelBase(object):
     def simple_search(cls, search_string):
         filters = None
 
-        if cls.simple_search_config:
+        if cls.__simple_search__:
             terms = [s for s in search_string.split()]
-            fields = cls.simple_search_config.keys()
+            fields = cls.__simple_search__.keys()
             field_count = len(fields)
 
             search_filters = []
@@ -139,7 +202,7 @@ class ModelBase(object):
                 # create a dict with each `config_search` key and `term` so filters can be applied to each combination
                 # i.e. { config_search_key1: term, config_search_key2: term, ..., config_search_keyN, term }
                 search_dict = dict(zip(fields, [term]*field_count))
-                term_filters = cls.get_search(search_dict, cls.simple_search_config)
+                term_filters = cls.get_search(search_dict, cls.__simple_search__)
 
                 if term_filters:
                     # `or` filters together since only 1 filter needs to match for `term`
@@ -228,7 +291,7 @@ class QueryProperty(object):
             return None
 
 def make_declarative_base(session=None, query_property=None, Model=None):
-    Model = Model or declarative_base(cls=ModelBase, constructor=ModelBase.__init__)
+    Model = Model or declarative_base(cls=ModelBase, constructor=ModelBase.__init__, metaclass=ModelMeta)
     extend_declarative_base(Model, session, query_property)
     return Model
 
@@ -237,3 +300,15 @@ def extend_declarative_base(Model, session=None, query_property=None):
     if session:
         Model.query = query_property(session) if query_property else QueryProperty(session)
 
+def event(event_name, **kargs):
+    def _event(f, *args, **kargs):
+        f.__event__ = Event(event_name, f, kargs)
+
+        @functools.wraps(f)
+        def wrapper(cls, *args, **kargs):
+            return f(*args, **kargs)
+
+        return wrapper
+
+    return _event
+
