@@ -1,6 +1,9 @@
 
+import re
 from collections import Iterable
 
+from sqlalchemy import Column
+
 class classproperty(object):
     '''
     Decorator that adds class properties.
@@ -17,3 +20,13 @@ def is_sequence(obj):
     Test if `obj` is an iterable but not dict or string
     '''
     return isinstance(obj, Iterable) and not isinstance(obj, basestring) and not isinstance(obj, dict)
+
+def has_primary_key(obj):
+    return any(v.primary_key for k,v in obj.iteritems() if isinstance(v, Column))
+
+first_cap_re = re.compile('(.)([A-Z][a-z]+)')
+all_cap_re = re.compile('([a-z0-9])([A-Z])')
+
+def camelcase_to_underscore(name):
+    s1 = first_cap_re.sub(r'\1_\2', name)
+    return all_cap_re.sub(r'\1_\2', s1).lower()
